"use strict";

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _typeof(obj) { return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj; }

var _require = require("events");

var EventEmitter = _require.EventEmitter;

var invariant = require("invariant");

var _uid = 1;
var names = {};
var shaders = {};

var Shaders = _extends({
  create: function create(obj) {
    invariant((typeof obj === "undefined" ? "undefined" : _typeof(obj)) === "object", "config must be an object");
    var result = {};
    for (var key in obj) {
      var shader = obj[key];
      invariant((typeof shader === "undefined" ? "undefined" : _typeof(shader)) === "object" && typeof shader.frag === "string", "invalid shader given to Shaders.create(). A valid shader is a { frag: String }");
      var id = _uid++;
      if (!shader.name) shader.name = key;
      names[id] = shader.name;
      shaders[id] = shader;
      this.emit("add", id, shader);
      result[key] = id;
    }
    return result;
  },
  remove: function remove(id) {
    invariant(id in shaders, "There is no such shader '%s'", id);
    delete shaders[id];
    delete names[id];
    this.emit("remove", id);
  },
  get: function get(id) {
    return shaders[id];
  },
  getName: function getName(id) {
    return names[id];
  },
  list: function list() {
    return Object.keys(shaders);
  },
  exists: function exists(id) {
    return typeof id === "number" && id >= 1 && id < _uid;
  }
}, EventEmitter.prototype);

EventEmitter.call(Shaders);

module.exports = Object.freeze(Shaders);
//# sourceMappingURL=Shaders.js.map